package com.ruoyi.web.controller.timer;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;
import org.springframework.scheduling.support.CronTrigger;

import java.util.Date;


/**
 * 将定时任务继承该类
 * * 第一位，表示秒，取值0-59
 * * 第二位，表示分，取值0-59
 * * 第三位，表示小时，取值0-23
 * * 第四位，日期天/日，取值1-31
 * * 第五位，日期月份，取值1-12
 * * 第六位，星期，取值1-7，星期一，星期二...，注：不是第1周，第二周的意思
 *           另外：1表示星期天，2表示星期一。
 * (*)星号：可以理解为每的意思，每秒，每分，每天，每月，每年...
 * (?)问号：问号只能出现在日期和星期这两个位置。
 * (-)减号：表达一个范围，如在小时字段中使用“10-12”，则表示从10到12点，即10,11,12
 * (,)逗号：表达一个列表值，如在星期字段中使用“1,2,4”，则表示星期一，星期二，星期四
 * (/)斜杠：如：x/y，x是开始值，y是步长，比如在第一位（秒） 0/15就是，从0秒开始，每15秒，最后就是0，15，30，45，60
0 0 3 * * ?     每天3点执行
0 5 3 * * ?     每天3点5分执行
0 5 3 ? * *     每天3点5分执行，与上面作用相同
0 5/10 3 * * ?  每天3点的 5分，15分，25分，35分，45分，55分这几个时间点执行
0 10 3 ? * 1    每周星期天，3点10分 执行，注：1表示星期天
0 10 3 ? * 1#3  每个月的第三个星期，星期天 执行，#号只能出现在星期的位置
 *
 **/
public abstract class BaseTask implements SchedulingConfigurer {
    private static Logger logger = LoggerFactory.getLogger(BaseTask.class);
//    private String type = "DynamicScheduledTask";
    private String pattern;

    @Override
    public void configureTasks(ScheduledTaskRegistrar scheduledTaskRegistrar) {
        logger.info("DynamicScheduledTask====run:" + System.currentTimeMillis());
        scheduledTaskRegistrar.addTriggerTask(()->{
            run();
        },(triggerContext)->{
            CronTrigger cronTrigger = new CronTrigger(getPattern());
            Date nextExecDate = cronTrigger.nextExecutionTime(triggerContext);
            return nextExecDate;
        });
    }

    /**
     * 将执行的方法
     */
    protected abstract void run();
    /**
     * create by xqh on 2019/6/26 13:45
     * 需要定义规则：pattern
     */
    protected abstract String getPattern();
}


